import "./root";
import * as _a from "../core/dom";
_a.styles.append(".bk-root .bk-shading {\n  position: absolute;\n  display: block;\n  border: 1px dashed green;\n}\n");
export const bk_annotation = "bk-annotation";
export const bk_shading = "bk-shading";
export const bk_annotation_child = "bk-annotation-child";
//# sourceMappingURL=annotations.js.map