export declare const svg_colors: {
    indianred: string;
    lightcoral: string;
    salmon: string;
    darksalmon: string;
    lightsalmon: string;
    crimson: string;
    red: string;
    firebrick: string;
    darkred: string;
    pink: string;
    lightpink: string;
    hotpink: string;
    deeppink: string;
    mediumvioletred: string;
    palevioletred: string;
    coral: string;
    tomato: string;
    orangered: string;
    darkorange: string;
    orange: string;
    gold: string;
    yellow: string;
    lightyellow: string;
    lemonchiffon: string;
    lightgoldenrodyellow: string;
    papayawhip: string;
    moccasin: string;
    peachpuff: string;
    palegoldenrod: string;
    khaki: string;
    darkkhaki: string;
    lavender: string;
    thistle: string;
    plum: string;
    violet: string;
    orchid: string;
    fuchsia: string;
    magenta: string;
    mediumorchid: string;
    mediumpurple: string;
    blueviolet: string;
    darkviolet: string;
    darkorchid: string;
    darkmagenta: string;
    purple: string;
    indigo: string;
    slateblue: string;
    darkslateblue: string;
    mediumslateblue: string;
    greenyellow: string;
    chartreuse: string;
    lawngreen: string;
    lime: string;
    limegreen: string;
    palegreen: string;
    lightgreen: string;
    mediumspringgreen: string;
    springgreen: string;
    mediumseagreen: string;
    seagreen: string;
    forestgreen: string;
    green: string;
    darkgreen: string;
    yellowgreen: string;
    olivedrab: string;
    olive: string;
    darkolivegreen: string;
    mediumaquamarine: string;
    darkseagreen: string;
    lightseagreen: string;
    darkcyan: string;
    teal: string;
    aqua: string;
    cyan: string;
    lightcyan: string;
    paleturquoise: string;
    aquamarine: string;
    turquoise: string;
    mediumturquoise: string;
    darkturquoise: string;
    cadetblue: string;
    steelblue: string;
    lightsteelblue: string;
    powderblue: string;
    lightblue: string;
    skyblue: string;
    lightskyblue: string;
    deepskyblue: string;
    dodgerblue: string;
    cornflowerblue: string;
    royalblue: string;
    blue: string;
    mediumblue: string;
    darkblue: string;
    navy: string;
    midnightblue: string;
    cornsilk: string;
    blanchedalmond: string;
    bisque: string;
    navajowhite: string;
    wheat: string;
    burlywood: string;
    tan: string;
    rosybrown: string;
    sandybrown: string;
    goldenrod: string;
    darkgoldenrod: string;
    peru: string;
    chocolate: string;
    saddlebrown: string;
    sienna: string;
    brown: string;
    maroon: string;
    white: string;
    snow: string;
    honeydew: string;
    mintcream: string;
    azure: string;
    aliceblue: string;
    ghostwhite: string;
    whitesmoke: string;
    seashell: string;
    beige: string;
    oldlace: string;
    floralwhite: string;
    ivory: string;
    antiquewhite: string;
    linen: string;
    lavenderblush: string;
    mistyrose: string;
    gainsboro: string;
    lightgray: string;
    lightgrey: string;
    silver: string;
    darkgray: string;
    darkgrey: string;
    gray: string;
    grey: string;
    dimgray: string;
    dimgrey: string;
    lightslategray: string;
    lightslategrey: string;
    slategray: string;
    slategrey: string;
    darkslategray: string;
    darkslategrey: string;
    black: string;
};
export declare type SVGColor = keyof typeof svg_colors;
export declare function is_svg_color(color: string): color is SVGColor;
