import { Scale } from "./scale";
export class ContinuousScale extends Scale {
    constructor(attrs) {
        super(attrs);
    }
}
ContinuousScale.__name__ = "ContinuousScale";
//# sourceMappingURL=continuous_scale.js.map