import "../root";
import "./nouislider";
import * as _a from "../../core/dom";
_a.styles.append(".bk-root .bk-slider-title {\n  white-space: nowrap;\n}\n.bk-root .bk-slider-value {\n  font-weight: 600;\n}\n");
export const bk_slider_value = "bk-slider-value";
export const bk_slider_title = "bk-slider-title";
export const bk_input_group = "bk-input-group";
//# sourceMappingURL=sliders.js.map